package com.alterdekim.flash.decompiler.util;

import lombok.extern.slf4j.Slf4j;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

@Slf4j
public class ByteUtil {
    public static int bytesToInt(byte[] bytes) {
        int value = 0;
        for (byte b : bytes) {
            value = (value << 8) + (b & 0xFF);
        }
        return value;
    }

    public static int bytesToIntRev(byte[] bytes) {
        int value = 0;
        for(int i = bytes.length-1; i >= 0; i-- ) {
            value = (value << 8) + (bytes[i] & 0xFF);
        }
        return value;
    }

    private static byte[] int2bytes(int val, int len) {
        byte[] r = new byte[len];
        for( int i = 0; i < len; i++ ) {
            r[i] = (byte) ( (val >> 8*i) & 0xFF );
        }
        return r;
    }

    public static List<Byte> intToBytes(int i) {
        return fromPrimitive(int2bytes(i, 2));
    }

    public static List<Byte> intToBytes8(int i) {
        return fromPrimitive(int2bytes(i, 1));
    }

    public static List<Byte> intToBytes32(int i) {
        return fromPrimitive(int2bytes(i, 4));
    }

    public static List<Byte> fromPrimitive(byte[] b) {
        return IntStream.range(0, b.length).boxed().map(i -> b[i]).collect(Collectors.toList());
    }

    public static byte[] toPrimitive(List<Byte> b) {
        byte[] a = new byte[b.size()];
        IntStream.range(0, b.size()).boxed().forEach(i -> a[i] = b.get(i));
        return a;
    }

    public static double bytesToDouble(byte[] bytes) {
        byte[] b1 = new byte[8];
        for( int i = bytes.length-1, u = 0; i >= 0; i--, u++ ) {
            b1[u] = bytes[i];
        }
        byte[] d = new byte[8];
        System.arraycopy(Arrays.copyOfRange(b1, 4, 8), 0, d, 0, 4);
        System.arraycopy(Arrays.copyOfRange(b1, 0, 4), 0, d, 4, 4);
        return ByteBuffer.wrap(d).order(ByteOrder.BIG_ENDIAN).getDouble();
    }

    public static List<Byte> doubleToBytes(double d) {
        byte[] b = ByteBuffer.allocate(8).order(ByteOrder.LITTLE_ENDIAN).putDouble(d).array();
        byte[] b1 = new byte[8];
        System.arraycopy(b, 4, b1, 0, 4);
        System.arraycopy(b, 0, b1, 4, 4);
        return fromPrimitive(b1);
    }
}
